
/******************************************************************************
* MODULE     : associator.gen.cc
* DESCRIPTION: used for translations and analysing text
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <analyze.gen.h>
#include <file.gen.h>
#include <associator.gen.h>
#include <list.gen.cc>
#include <hashmap.gen.cc>

#module code_associator
#import analyze
#import file
#import associator
#import code_list (path)
#import code_hashmap (path, list<path>)
#import code_resource (associator_rep, associator)

/******************************************************************************
* Associater initialization
******************************************************************************/

associator_rep::associator_rep (string s):
  rep<associator> (s), name (1), code (0), var (0), ass (links ())
{
  name[0]= "?";
  new_var ("LANGUAGE");
  new_var ("DUTCH");
  new_var ("ENGLISH");
  new_var ("FRENCH");
  new_var ("GERMAN");
  new_var ("MEANS");
  new_var ("WRITES");

  (void) execute ("(LANGUAGE DUTCH)");
  (void) execute ("(LANGUAGE ENGLISH)");
  (void) execute ("(LANGUAGE FRENCH)");
  (void) execute ("(LANGUAGE GERMAN)");

  new_var ("CLASS");
  new_var ("INTERFACE");
  new_var ("MATHEMATICS");
}

/******************************************************************************
* Introducing new concepts and links
******************************************************************************/

concept
associator_rep::new_var (string s) {
  var (s)= N(name);
  name << "*";
  return var [s];
}

concept
associator_rep::new_string (string s) {
  if (!code->contains (s)) {
    code (s)= N(name);
    name << s;
  }
  return code [s];
}

void
associator_rep::insert (link lk) {
  link done;
  while (TRUE) {
    if (!ass->contains (done)) ass (done)= links ();
    ass (done)= links (lk, ass (done));
    if (nil (lk)) return;
    done= path_down (done, lk->item);
    lk  = lk->next;
  }
}

/******************************************************************************
* Subroutines
******************************************************************************/

concept
associator_rep::get_concept (tree t) {
  if (N(t)>0) return 0;
  string s= t->label;
  if (is_string (s)) return new_string (s (1, N(s)-1));
  else return var[s];
}

static int
search (tree vars, tree which) {
  int i;
  for (i=0; i<N(vars); i++)
    if (vars[i] == which) return i;
  fatal_error ("variable not found", "search", "associator.gen.cc");
}

/******************************************************************************
* Conditional execution
******************************************************************************/
  
links
associator_rep::condition (tree vars, link vals, tree t) {
  // cout << "Condition " << vars << ", " << vals << ", " << t << "\n";

  int i, j;
  link start;

  for (i=0; i<N(t); i++) {
    concept con= get_concept (t[i]);
    if (con == -1) {
      con= vals [search (vars, t[i])];
      if (con == 0) break;
      else start= path_down (start, con);
    }
    else start= path_down (start, con);
  }

  links lks= ass [start];
  links new_vals;

  for (; !nil (lks); lks=lks->next) {
    link vs= copy (vals);
    link lk= lks->item;
    if (N(lk)+i != N(t)) continue;
    for (j=i; j<N(t); j++) {
      concept con= get_concept (t[j]);
      if (con == -1) {
	con= vals [search (vars, t[j])];
	if (con == 0) {
	  con= lk [j-i];
	  vs [search (vars, t[j])]= con;
	}
      }
      if (con != lk [j-i]) break;
    }
    if (j==N(t)) new_vals= links (vs, new_vals);
  }

  return new_vals;
}

links
associator_rep::execute_conditional (tree vars, tree t) {
  // cout << "Conditional " << vars << ", " << t << "\n";

  int i, n= N(vars);
  link lk;
  for (i=0; i<n; i++) lk= link (0, lk); 
  for (i=0; i<n; i++) var (vars[i]->label)= -1;
  links lks (lk);

  for (i=0; i<N(t); i++) {
    if (!is_func (t[i], "")) continue;
    links counter= lks;
    links new_lks;
    while (!nil (counter)) {
      links r= condition (vars, counter->item, t[i]);
      while (!nil (r)) {
	new_lks= links (r->item, new_lks);
	r= r->next;
      }
      counter= counter->next;
    }
    lks= new_lks;
  }

  for (i=0; i<n; i++) var (vars[i]->label)= 0;
  return lks;
}

/******************************************************************************
* Declarative execution
******************************************************************************/

concepts
associator_rep::execute_declarative (tree t) {
  // cout << "Execute: " << t << "\n";

  if (t->label == "decls") {
    int i;
    concepts cons;
    for (i=0; i<N(t); i++)
      cons= execute_declarative (t[i]);
    return cons;
  }

  if (N(t)==0) return get_concept (t->label);

  if (is_func (t, "new", 1)) {
    return concepts (new_var (t[0]->label));
  }

  if (is_func (t, "")) {
    int i;
    link lk;
    for (i=N(t)-1; i>=0; i--)
      lk= link (get_concept (t[i]), lk);
    insert (lk);
    return concepts ();
  }

  if (is_func (t, "for", 3)) {
    links lks= execute_conditional (t[0], t[1]);
    concepts cons;
    while (!nil (lks)) {
      int  i = 0;
      link lk= lks->item;
      while (!nil (lk)) {
	var (t[0][i]->label)= lk->item;
	i++;
	lk= lk->next;
      }
      cons= execute_declarative (t[2]);
      lks = lks->next;
    }
    return cons;
  }
  
  if (is_func (t, "print", 1)) {
    concepts cons= execute_declarative (t[0]);
    if (N(cons)>1) cerr << "TeXmacs] Ambiguous expression to print\n";
    if (!nil (cons)) cout << name [cons->item] << "\n";
    return cons;
  }

  return concepts ();
}

/******************************************************************************
* Parsing
******************************************************************************/

void
associator_rep::skip_spaces () {
  while ((pos<N(s)) && (s[pos]==' ') || (s[pos]=='\t') || (s[pos]=='\n'))
    pos++;
}

string
associator_rep::read_string () {
  int start= pos;
  pos++;
  while ((pos<N(s)) && (s[pos]!=((char) 34))) pos++;
  if (pos<N(s)) pos++;
  string r= s (start, pos);
  skip_spaces ();
  return r;
}

string
associator_rep::read_id () {
  int start= pos;
  while ((pos<N(s)) && (s[pos]!=' ') && (s[pos]!='\t') &&
	 (s[pos]!='\n') && (s[pos]!='(') && (s[pos]!=')')) pos++;
  string r= s (start, pos);
  skip_spaces ();
  return r;
}

bool
associator_rep::upcoming (string next) {
  if ((pos+N(next) <= N(s)) && (s(pos,pos+N(next)) == next)) {
    pos += N(next);
    skip_spaces ();
    return TRUE;
  }
  else return FALSE;
}

tree
associator_rep::parse () {
  skip_spaces ();

  if (upcoming ("new"))
    return tree ("new", parse ());
  if (upcoming ("print"))
    return tree ("print", parse ());
  if (upcoming ("return"))
    return tree ("return", parse ());
  if (upcoming ("(")) {
    tree t ("");
    while ((pos<N(s)) && (!upcoming (")"))) t << parse ();
    return t;
  }
  if (upcoming ("for")) {
    tree vars ("vars"), conds ("conds"), decls ("decls");
    while ((pos<N(s)) && (!upcoming ("with"))) vars << parse ();
    while ((pos<N(s)) && (!upcoming ("do"))) conds << parse ();
    while ((pos<N(s)) && (!upcoming ("od"))) decls << parse ();
    return tree ("for", vars, conds, decls);
  }

  if (pos==N(s)) return "";
  if (s[pos]==((char) 34)) return read_string ();
  return read_id ();
}

/******************************************************************************
* Interface
******************************************************************************/

void
associator_rep::load (string fname) {
  if (loaded_files->contains (fname)) return;
  else loaded_files->insert (fname);

  string s;
  fname= fname * ".ass";
  if (debug (0)) cout << "TeXmacs] Loading " << fname << "\n";
  load_string ("$TEXMACS_PATH/data/ass", fname, s, TRUE);
  execute (s);
}

string
associator_rep::meaning (concept con) {
  return name [con];
}

concepts
associator_rep::execute (string prg) {
  s  = prg;
  pos= 0;
  tree t ("decls");
  while (pos<N(s)) t << parse ();
  return execute_declarative (t);
}

associator
new_associator (string name) {
  if (resource<associator> -> contains (name))
    return associator (name);
  return new associator_rep (name);
}

associator std_associator= new_associator ("std");

/******************************************************************************
* Translation routines
******************************************************************************/

string
associator_rep::translate (string s, string source_lan, string dest_lan) {
  if ((N(s)==0) || (source_lan==dest_lan)) return s;

  bool flag= is_upcase (s[0]);
  string ss= locase_first (s);
  tree prg ("for",
	    tree ("vars", "c", "t"),
	    tree ("conds",
		  tree ("", "MEANS", source_lan, "\"" * ss * "\"", "c"),
		  tree ("", "WRITES", dest_lan, "c", "t")),
	    tree ("decls", "t"));
  concepts cons= std_associator->execute_declarative (prg);

  if (N (cons) == 0) return s;
  string r= std_associator->meaning (cons->item);
  if (flag) return upcase_first (r);
  else return r;
}

#endmodule // associator
