
/******************************************************************************
* MODULE     : safe_real.gen.cc
* DESCRIPTION: safe_reals based on gmp
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Arithmetic/safe_real.gen.h>

#module code_safe_real
#import safe_real
#include <math.h>

/******************************************************************************
* conversion
******************************************************************************/

static int safe_real_precision=0;

void
set_safe_real_precision (int bits) {
  safe_real_precision= bits;
  mpfe_set_precision (bits);
}

safe_real::safe_real () {
  rep= new safe_real_rep();
}

safe_real::safe_real (int i) {
  rep= new safe_real_rep();
  mpfe_set_si (rep->F, i);
}

safe_real::operator string () {
  mps_t temp;
  mps_init (temp);
  mps_set_fe (temp, rep->F);
  string s (temp->s, temp->len);
  mps_clear (temp);
  return s;
}

safe_real::operator tree () {
  return tree ((string) (*this));
}

ostream&
operator << (ostream& out, safe_real x) {
  return out << ((string) x);
}

/******************************************************************************
* basic arithmetics
******************************************************************************/

safe_real
operator + (safe_real x, safe_real y) {
  safe_real z;
  mpfe_add (z->F, x->F, y->F);
  return z;
}

safe_real
operator - (safe_real x) {
  safe_real y;
  mpfe_neg (y->F, x->F);
  return y;
}

safe_real
operator - (safe_real x, safe_real y) {
  safe_real z;
  mpfe_sub (z->F, x->F, y->F);
  return z;
}

safe_real
operator * (safe_real x, safe_real y) {
  safe_real z;
  mpfe_mul (z->F, x->F, y->F);
  return z;
}

safe_real
operator / (safe_real x, safe_real y) {
  safe_real z;
  mpfe_div (z->F, x->F, y->F);
  return z;
}

#endmodule // code_safe_real
