
/******************************************************************************
* MODULE     : safe_real.gen.h
* DESCRIPTION: safe reals based on gmpx
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tree.gen.h>

#module safe_real
#import tree
#include <gmpx.h>

struct safe_real_rep: public concrete_struct {
  mpfe_t F;
  inline safe_real_rep () { mpfe_init (F); }
  inline ~safe_real_rep () { mpfe_clear (F); }
};

class safe_real {
#import concrete (safe_real, safe_real_rep)
public:
  safe_real ();
  safe_real (int i);
  operator string ();
  operator tree ();
};
#import code_concrete (safe_real, safe_real_rep)

void       set_safe_real_precision (int bits);
ostream&   operator << (ostream& out, safe_real x);
safe_real  operator + (safe_real x, safe_real y);
safe_real  operator - (safe_real x);
safe_real  operator - (safe_real x, safe_real y);
safe_real  operator * (safe_real x, safe_real y);
safe_real  operator / (safe_real x, safe_real y);

inline void operator += (safe_real& x, safe_real y) { x=x+y; }
inline void operator -= (safe_real& x, safe_real y) { x=x-y; }
inline void operator *= (safe_real& x, safe_real y) { x=x*y; }
inline void operator /= (safe_real& x, safe_real y) { x=x/y; }

#endmodule // safe_real
