
/******************************************************************************
* MODULE     : series.gen.h
* DESCRIPTION: abstract power series
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Univariate/zealous.gen.h>
#include <tree.gen.h>

#module series (C)
#import zealous (C)
#import tree

#define POLYNOMIAL   0
#define RATIONAL     1
#define ALGEBRAIC    2
#define HOLONOMIC    3
#define ZEALOUS      4
#define RELAXED      5

class series_rep<C>: public abstract_struct {
protected:
  int        type;   // one of the above types
  int        n;      // number of computed terms
  zealous<C> value;  // truncated series with computed terms

  inline series_rep<C> (int type2): type (type2), n (0), value (0) {}
  inline virtual ~series_rep<C> () {}
  virtual C next () = 0;
  virtual operator tree () = 0;

public:
  inline virtual void set_precision (int terms) { (void) terms; }
  friend class series<C>;
};

struct series<C> {
#import abstract_null (series<C>, series_rep<C>)
  series<C> (C c, int deg=0);
  series<C> (zealous<C> s);
  series<C> (C (*coeffs) (int));
  C operator [] (int n);
  zealous<C> operator () (int start, int end);
  operator tree ();
};
#import code_abstract_null (series<C>, series_rep<C>)

series<C> operator >> (series<C> f, int n);
series<C> operator << (series<C> f, int n);
series<C> operator +  (series<C> f, series<C> g);
series<C> operator -  (series<C> f, series<C> g);
series<C> operator *  (C c, series<C> f);
series<C> operator *  (series<C> f, series<C> g);
series<C> operator /  (series<C> f, series<C> g);

series<C> d (series<C> f);
#ifdef has_divisible<C>
series<C> operator / (series<C> f, C c);
series<C> I (series<C> f);
series<C> log (series<C> f);
series<C> exp (series<C> f);
#endif

series<C> Mahler  (series<C> f, int p);
series<C> compose (series<C> f, zealous<C> g);
series<C> compose (series<C> f, zealous<C> num, zealous<C> den);
series<C> compose (series<C> f, series<C> g);
series<C> reverse (series<C> f);

void set_series<C>_precision (int n);
ostream& operator << (ostream& out, series<C> f);

#endmodule // series (C)
